-- POSLIB Test Functions
-- Tests poslib implementation
-- This file contains functions required by pt.lua
--
-- 1997-8 Claudio Terra
-- PUC-Rio

function TestDateTable(t)
	assert(type(t)=="table")
	assert(type(t.sec)=="number")
	assert(t.sec <= 59 and t.sec >= 0)
	assert(type(t.min)=="number")
	assert(t.min <= 59 and t.min >= 0)
	assert(type(t.hour)=="number")
	assert(t.hour <= 23 and t.hour >= 0)
	assert(type(t.day)=="number")
	assert(t.day <= 31 and t.day >= 1)
	assert(type(t.month)=="number")
	assert(t.month <= 12 and t.month >= 1)
end

function TestDirTable(t)
	assert(type(t)=="table")
	assert(t.n ~= nil)
	local i=t.n
	assert(t[i] ~= nil)
end

function TestPathconf2(path)
	local t = pathconf(path)
	assert(type(t)=="table")

	if t.link_max == nil then print("  No link_max!");
	else
		assert(t.link_max == pathconf(path, "link_max"))
		assert(type(t.link_max)=="number")
	end

	if t.max_canon==nil then print("  No max_canon!");
	else
		assert(t.max_canon == pathconf(path, "max_canon")) 
		assert(type(t.max_canon)=="number")
	end

	if t.max_input==nil then print("  No max_input!");
	else
		assert(t.max_input == pathconf(path, "max_input"))
		assert(type(t.max_input)=="number")
	end
	
	if t.name_max==nil then print("  No name_max!");
	else
		assert(t.name_max == pathconf(path, "name_max"))
		assert(type(t.name_max)=="number")
	end
	
	if t.path_max==nil then print("  No path_max!");
	else
		assert(t.path_max == pathconf(path, "path_max"))
		assert(type(t.path_max)=="number")
	end

	if t.pipe_buf==nil then print("  No pipe_buf!");
	else
		assert(t.pipe_buf == pathconf(path, "pipe_buf"))
		assert(type(t.pipe_buf)=="number")
	end

	if t.chown_restricted==nil then print("  No chown_restricted!");
	else
		assert(t.chown_restricted == pathconf(path, "chown_restricted"))
		assert(type(t.chown_restricted)=="number")
	end

	if t.no_trunc==nil then print("  No no_trunc!");
	else
		assert(t.no_trunc == pathconf(path, "no_trunc"))
		assert(type(t.no_trunc)=="number")
	end

	if t.vdisable==nil then print("  No vdisable!");
	else
		assert(t.vdisable == pathconf(path, "vdisable"))
		assert(type(t.vdisable)=="number")
	end
end

function TestStat2(path)
	local t = stat(path)
	assert(type(t)=="table")
	assert(t.mode == stat(path, "mode"))
	assert(type(t.mode)=="string")
	
	assert(t.ino == stat(path, "ino"))
	assert(type(t.ino)=="number")
	
	assert(t.dev == stat(path, "dev"))
	assert(type(t.dev)=="number")
	
	assert(t.nlink == stat(path, "nlink"))
	assert(type(t.nlink)=="number")
	
	assert(t.uid == stat(path, "uid"))
	assert(type(t.uid)=="number")
	
	assert(t.gid == stat(path, "gid"))
	assert(type(t.gid)=="number")
	
	assert(t.size == stat(path, "size"))
	assert(type(t.size)=="number")
	
	TestDateTable(t.atime)
	TestDateTable(stat(path, "atime"))
	
	TestDateTable(t.mtime)
	TestDateTable(stat(path, "mtime"))

	TestDateTable(t.ctime)
	TestDateTable(stat(path, "ctime"))
end

function TestUmask()
	local orig=umask();
	assert(umask("rwxr-xr-x") == "rwxr-xr-x")
	assert(umask("u-xr") == "-w-r-xr-x")
	assert(umask("u-rwx, u+rwx") == "rwxr-xr-x")
	assert(umask("rwxr-xr-x") == "rwxr-xr-x")
	assert(umask() == umask("a="))
	assert(umask() == umask("a-"))
	assert(umask() == umask("a+"))
	umask("r--------")
	assert(umask() == "r--------")
	assert(umask("u+w, g+w, o+r") == "rw--w-r--")
	-- note the order of processing of modemuncher:
	assert(umask("u+x, u-x") == "rw--w-r--")
	assert(umask("u-x, u+x") == "rwx-w-r--")
	umask(orig)
	assert(orig == umask())
end

function TestGetenv()
	local vars, n = getenv()
	local c=next(vars, nil)
	assert(vars[c] == getenv(c))
end

function TestPathconf()
	local filename=tmpname() -- get a unique name
	local filehandle=writeto(filename) -- open a file
	assert(filehandle ~= nil) -- file opened succesfully
	assert(pathconf(filehandle, "link_max") == pathconf(filename, "link_max"))
	TestPathconf2(filename)
	writeto() -- closes the file
	assert(remove(filename) ~= nil)
end

function TestStat()
	local filename=tmpname() -- get a unique name
	local filehandle=writeto(filename) -- open a file
	assert(filehandle ~= nil) -- file opened succesfully
	assert(stat(filehandle, "mode") == stat(filename, "mode"))
	TestStat2(filename)
	writeto() -- closes the file
	assert(remove(filename) ~= nil)
end

function TestLock()
	local filename=tmpname() -- get a unique name
	local filehandle=writeto(filename) -- open a file for writing
	assert(filehandle ~= nil) -- file opened succesfully
	assert(lock(filehandle, "w?") == nil) -- no lock yet
	-- hard to test lock from a single process
	writeto() -- closes the file
	assert(remove(filename) ~= nil)
end

function TestModes()
	local filename=tmpname() -- get a unique name
	local filehandle=writeto(filename) -- open a file for writing
	local t = getpasswd(getprocessid("uid"))
	
	assert(filehandle ~= nil) -- file opened succesfully
	assert(access(filename) ~= nil) -- file exists
	
	chmod(filename, "rwxr-xr-x")
	assert(stat(filename, "mode") == "rwxr-xr-x")
	assert(access(filename, "rwx") ~= nil) -- file can be r, w, x
	
	chmod(filename, "u-xr")
	assert(stat(filename, "mode") == "-w-r-xr-x")
	assert(access(filename, "w") ~= nil)
	
	chmod(filename, "u-rwx, u+rwx")
	assert(stat(filename, "mode") == "rwxr-xr-x")
	
	chmod(filename, "rwxr-xr-x")
	assert(stat(filename, "mode") == "rwxr-xr-x")
	
	chmod(filename, "r--------")
	assert(access(filename, "r") ~= nil)
	if t.uid ~= 0 then -- root is uid 0
		assert(access(filename, "wx") == nil)
		-- this test makes no sense under root power
		else print ("You're under UID 0 (root) - access won't fail.")
	end
	
	chmod(filename, "u+w, g+w, o+r")
	assert(stat(filename, "mode") == "rw--w-r--")
	
	chmod(filename, "u+x, u-x")
	assert(stat(filename, "mode") == "rw--w-r--")
	
	chmod(filename, "u-x, u+x")
	assert(stat(filename, "mode") == "rwx-w-r--")
	
	writeto() -- closes the file
	assert(remove(filename) ~= nil)
end

function TestUtime()
	local filename=tmpname() -- get a unique name
	local filehandle=writeto(filename) -- open a file for writing
	assert(filehandle ~= nil) -- file opened succesfully
	assert(access(filename) ~= nil) -- file exists
	chmod(filename, "rwxr--r--")
	local somedate={}
	somedate.year=1984
	somedate.month=1
	somedate.day=24
	
	local today=currdate()
	TestDateTable(today)
	utime(filename, today, somedate)
	local actime=stat(filename, "atime")
	TestDateTable(actime)
	
	local modtime=stat(filename, "mtime")
	TestDateTable(modtime)

	assert(today.year == actime.year)
	assert(today.month == actime.month)
	assert(today.day == actime.day)
	
	assert(somedate.year == modtime.year)
	assert(somedate.month == modtime.month)
	assert(somedate.day == modtime.day)

	writeto() -- closes the file
	assert(remove(filename) ~= nil)
end

function TestMkdir()
	local orig=umask()
	local filename=tmpname() -- get a unique name
	mkdir(filename) -- creates new dir; lua_error on failure
	assert(access(filename) ~= nil) -- file exists
	assert(stat(filename, "mode") == orig) -- mode of new dir == umask
	local currdir=getcwd()
	chdir(filename)
	local dot=getcwd()
	--assert(dot==filename) -- may fail on SunOS 4
	chdir(currdir)
	rmdir(filename)
end

function TestGetprocessid()
	local t = getprocessid()
	assert(type(t)=="table")
	assert(t.egid == getprocessid("egid"))
	assert(type(t.egid)=="number")
	
	assert(t.euid == getprocessid("euid"))
	assert(type(t.euid)=="number")
	
	assert(t.gid == getprocessid("gid"))
	assert(type(t.gid)=="number")
	
	assert(t.uid == getprocessid("uid"))
	assert(type(t.uid)=="number")
	
	assert(t.pgrp == getprocessid("pgrp"))
	assert(type(t.pgrp)=="number")
	
	assert(t.pid == getprocessid("pid"))
	assert(type(t.pid)=="number")
	
	assert(t.ppid == getprocessid("ppid"))
	assert(type(t.ppid)=="number")
end

function TestGetpasswd()
	local t = getpasswd(getprocessid("uid"))
	assert(type(t)=="table")

	assert(type(t.name)=="string")
	assert(type(t.uid)=="number")
	assert(type(t.gid)=="number")
	assert(type(t.dir)=="string")
	assert(type(t.shell)=="string")
	
	local t2=getpasswd(t.name)
	assert(t2.uid==t.uid)
	t2=getpasswd(t.uid)
	assert(t2.name==t.name)
end

function TestGetgroup()
	local t = getgroup(getprocessid("gid"))
	assert(type(t)=="table")

	assert(type(t.name)=="string")
	assert(type(t.gid)=="number")
	assert(type(t.n)=="number")
	
	local t2=getgroup(t.name)
	assert(type(t2)=="table")
	
	assert(t2.gid==t.gid)
	t2=getgroup(t.gid)
	assert(t2.name==t.name)
end






